#include "grid_sync.h"

#include <hls_stream.h>
#include <stdint.h>
#include "ap_fixed.h"
#include "hls_math.h"

void abc2dq0(float A, float B, float C, float wt, float& d, float& q, float& zero)
{
#pragma HLS inline

	const ap_fixed<16,2> sqrt3_3 = 0.57733154296875;

	ap_fixed<32,16> A_fix = (ap_fixed<32,16>)A;
	ap_fixed<32,16> B_fix = (ap_fixed<32,16>)B;
	ap_fixed<32,16> C_fix = (ap_fixed<32,16>)C;
	ap_fixed<16,4> wt_fix = (ap_fixed<16,4>)wt;

	ap_fixed<32,16> beta_fix = (B_fix-C_fix)*sqrt3_3;
	ap_fixed<32,16> zero_fix = (A_fix+B_fix+C_fix)/3;
	ap_fixed<32,16> alpha_fix = A_fix-zero_fix;

	ap_fixed<16, 2> cos_wt = hls::cos(wt_fix);
	ap_fixed<16, 2> sin_wt = hls::sin(wt_fix);

	ap_fixed<32,16> d_fix = alpha_fix*cos_wt + beta_fix*sin_wt;
	ap_fixed<32,16> q_fix = -alpha_fix*sin_wt + beta_fix*cos_wt;

	d = (float)d_fix;
	q = (float)q_fix;
	zero = (float)zero_fix;
}

void pll(float Ugq, float Ts, float& theta, float& freq)
{
#pragma HLS inline

	const float theta_max = 6.283185307179586;
	const float theta_min = 0;

	static float pll_accum = 0;
#pragma HLS RESET variable=pll_accum

	float pll_kiTs = pll_ki*Ts;
	pll_accum += pll_kiTs* Ugq;
	float pi_out = pll_kp * Ugq + pll_accum;
	float w = pi_out + w0;

	static float wrapping_accum = 0;
#pragma HLS RESET variable=wrapping_accum

	wrapping_accum += Ts * w;

	if(wrapping_accum > theta_max)
	{
		wrapping_accum -= theta_max;
	}
	else if(wrapping_accum < theta_min)
	{
		wrapping_accum += theta_max;
	}

	theta = wrapping_accum;
	freq = w;
}

void vitis_grid_sync(	hls::stream<float>& in_Iga,
						hls::stream<float>& in_Igb,
						hls::stream<float>& in_Igc,
						hls::stream<float>& in_Uga,
						hls::stream<float>& in_Ugb,
						hls::stream<float>& in_Ugc,
						hls::stream<float>& in_Ts,
						hls::stream<float>& out_Igd,
						hls::stream<float>& out_Igq,
						hls::stream<float>& out_Ig0,
						hls::stream<float>& out_Ugd,
						hls::stream<float>& out_Ugq,
						hls::stream<float>& out_Ug0,
						hls::stream<float>& out_theta,
						hls::stream<float>& out_freq)
{
#pragma HLS INTERFACE axis port=in_Iga
#pragma HLS INTERFACE axis port=in_Igb
#pragma HLS INTERFACE axis port=in_Igc
#pragma HLS INTERFACE axis port=in_Uga
#pragma HLS INTERFACE axis port=in_Ugb
#pragma HLS INTERFACE axis port=in_Ugc
#pragma HLS INTERFACE axis port=in_Ts
#pragma HLS INTERFACE axis port=out_Igd
#pragma HLS INTERFACE axis port=out_Igq
#pragma HLS INTERFACE axis port=out_Ig0
#pragma HLS INTERFACE axis port=out_Ugd
#pragma HLS INTERFACE axis port=out_Ugq
#pragma HLS INTERFACE axis port=out_Ug0
#pragma HLS INTERFACE axis port=out_theta
#pragma HLS INTERFACE axis port=out_freq
#pragma HLS INTERFACE ap_ctrl_none port=return

	float Iga = in_Iga.read();
	float Igb = in_Igb.read();
	float Igc = in_Igc.read();
	float Uga = in_Uga.read();
	float Ugb = in_Ugb.read();
	float Ugc = in_Ugc.read();
	float Ts = in_Ts.read();

	float Igd,Igq,Ig0,Ugd,Ugq,Ug0,w;
	static float wt;
#pragma HLS RESET variable=wt

	abc2dq0(Uga, Ugb, Ugc, wt, Ugd, Ugq, Ug0);
	abc2dq0(Iga, Igb, Igc, wt, Igd, Igq, Ig0);
	pll(Ugq, Ts, wt, w);

	out_Igd.write(Igd);
	out_Igq.write(Igq);
	out_Ig0.write(Ig0);
	out_Ugd.write(Ugd);
	out_Ugq.write(Ugq);
	out_Ug0.write(Ug0);
	out_theta.write(wt);
	out_freq.write(w);
}
